import logging
import time
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class RedditIngestInput(BaseModel):
    subreddit_name: str = Field(description="Reddit subreddit name to be read and ingested into knowledge base")
    search: Optional[str] = Field(description="Text to search for within ingested posts", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of posts from Reddit subreddit to be read", default=10)

class RedditIngestTool(BaseTool):
    name: str = "Graphlit Reddit ingest tool"
    description: str = """Ingests posts from Reddit subreddit into knowledge base.
    Optionally accepts search text for searching within the ingested posts. If search text was not provided, all ingested posts will be returned.
    Returns extracted Markdown text and metadata from Reddit posts."""
    args_schema: Type[BaseModel] = RedditIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the RedditIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting posts. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, subreddit_name: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name=f'Reddit Feed [{subreddit_name}]',
                    type=enums.FeedTypes.REDDIT,
                    reddit=input_types.RedditFeedPropertiesInput(
                        subredditName=subreddit_name,
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, subreddit_name: str, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, subreddit_name, search, read_limit)



import logging
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class WebSearchInput(BaseModel):
    search: str = Field(description="Text to search for within web pages across the Internet")
    search_limit: Optional[int] = Field(description="Maximum number of web pages to be returned from web search", default=10)

class WebSearchTool(BaseTool):
    name: str = "Graphlit web search tool"
    description: str = """Accepts search query as string.
    Performs web search based on search query. Format the search query as what would be entered into a Google search.
    Returns URL, title and relevant Markdown text from resulting web pages."""
    args_schema: Type[BaseModel] = WebSearchInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, correlation_id: Optional[str] = None, **kwargs):
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.correlation_id = correlation_id

    async def _arun(self, search: str, search_limit: Optional[int] = None) -> Optional[str]:
        try:
            response = await self.graphlit.client.search_web(
                service=enums.SearchServiceTypes.TAVILY,
                text=search,
                limit=search_limit,
                correlation_id=self.correlation_id
            )

            results = response.search_web.results if response.search_web is not None else None

            if results is not None:
                logger.debug(f'Completed web search, found [{len(results)}] results.')

                return '\n\n'.join(f'URL: {result.uri}\nTitle: {result.title}\n\n{result.text}' for result in results) if results is not None else None
            else:
                return None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

    def _run(self, search: str, search_limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, search, search_limit)



import asyncio
from typing import Callable, Optional, List, Any, Coroutine
from graphlit_api import exceptions, input_types, enums
from .exceptions import ToolException

def format_person(person) -> List[str]:
    results = []

    results.append(f'**Person ID:** {person.id}')

    results.append(f'**Name:** {person.name}')

    if person.email is not None:
        results.append(f'**Email:** {person.email}')

    if person.uri is not None:
        results.append(f'**URI:** {person.uri}')

    if person.education is not None:
        results.append(f'**Education:** {person.education}')

    if person.occupation is not None:
        results.append(f'**Occupation:** {person.occupation}')

    results.append('\n')

    return results

def format_organization(organization) -> List[str]:
    results = []

    results.append(f'**Organization ID:** {organization.id}')

    results.append(f'**Name:** {organization.name}')

    if organization.email is not None:
        results.append(f'**Email:** {organization.email}')

    if organization.uri is not None:
        results.append(f'**URI:** {organization.uri}')

    results.append('\n')

    return results

def format_content(content, include_text: Optional[bool] = True) -> List[str]:
    results = []

    # Basic content details
    results.append(f"**Content ID:** {content.id}")

    if content.type == enums.ContentTypes.FILE:
        results.append(f"**File Type:** [{content.file_type}]")
        results.append(f"**File Name:** {content.file_name}")
    else:
        results.append(f"**Type:** [{content.type}]")
        if content.type not in [enums.ContentTypes.PAGE, enums.ContentTypes.EMAIL]:
            results.append(f"**Name:** {content.name}")

    # Optional metadata
    if content.uri:
        results.append(f"**URI:** {content.uri}")
    if content.creation_date:
        results.append(f"**Ingestion Date:** {content.creation_date}")
    if content.original_date:
        results.append(f"**Author Date:** {content.original_date}")

    # Issue details
    if content.issue:
        issue_attributes = [
            ("Title", content.issue.title),
            ("Identifier", content.issue.identifier),
            ("Type", content.issue.type),
            ("Project", content.issue.project),
            ("Team", content.issue.team),
            ("Status", content.issue.status),
            ("Priority", content.issue.priority),
        ]
        results.extend([f"**{label}:** {value}" for label, value in issue_attributes if value])

        if content.issue.labels:
            results.append(f"**Labels:** {', '.join(content.issue.labels)}")

    # Email details
    if content.email:
        email_attributes = [
            ("Subject", content.email.subject),
            ("Sensitivity", content.email.sensitivity.name if content.email.sensitivity else None),
            ("Priority", content.email.priority.name if content.email.priority else None),
            ("Importance", content.email.importance.name if content.email.importance else None),
            ("Labels", ', '.join(content.email.labels) if content.email.labels else None),
            ("To", ', '.join(f"{r.name} <{r.email}>" for r in content.email.to) if content.email.to else None),
            ("From", ', '.join(f"{r.name} <{r.email}>" for r in getattr(content.email, "from", []))),
            ("CC", ', '.join(f"{r.name} <{r.email}>" for r in content.email.cc) if content.email.cc else None),
            ("BCC", ', '.join(f"{r.name} <{r.email}>" for r in content.email.bcc) if content.email.bcc else None),
        ]
        results.extend([f"**{label}:** {value}" for label, value in email_attributes if value])

    # Document details
    if content.document:
        document_attributes = [
            ("Title", content.document.title),
            ("Author", content.document.author),
        ]
        results.extend([f"**{label}:** {value}" for label, value in document_attributes if value])

    # Audio details
    if content.audio:
        audio_attributes = [
            ("Title", content.audio.title),
            ("Host", content.audio.author),
            ("Episode", content.audio.episode),
            ("Series", content.audio.series),
        ]
        results.extend([f"**{label}:** {value}" for label, value in audio_attributes if value])

    # Image details
    if content.image:
        image_attributes = [
            ("Description", content.image.description),
            ("Software", content.image.software),
            ("Make", content.image.make),
            ("Model", content.image.model),
        ]
        results.extend([f"**{label}:** {value}" for label, value in image_attributes if value])

    # Links
    if content.links:
        if content.type in [enums.ContentTypes.PAGE]:
            results.extend([f"**{link.link_type} Link:** {link.uri}" for link in content.links[:100]])

    # Include text content if specified
    if include_text:
        if content.pages:
            for page in content.pages:
                if page.chunks:
                    results.append(f"**Page #{page.index + 1}:**")
                    results.extend([chunk.text for chunk in page.chunks])
                    results.append("\n---\n")

        if content.segments:
            for segment in content.segments:
                results.append(f"**Transcript Segment [{segment.start_time}-{segment.end_time}]:**")
                results.append(segment.text)
                results.append("\n---\n")

        if content.frames:
            for frame in content.frames:
                results.append(f"**Frame #{frame.index + 1}:**")
                results.append(frame.text)
                results.append("\n---\n")

        if not content.pages and not content.segments and not content.frames and content.markdown:
            results.append(content.markdown)
            results.append("\n")
    else:
        results.append("\n")

    return results

def run_async(coro_func: Callable[..., Coroutine[Any, Any, Any]], *args, **kwargs) -> Any:
    """
    Runs an async function synchronously, handling event loops properly.

    Args:
        coro_func: The asynchronous function to be run.
        *args: Positional arguments to pass to the async function.
        **kwargs: Keyword arguments to pass to the async function.

    Returns:
        The result of the async function execution.
    """
    try:
        loop = asyncio.get_event_loop()
        if loop.is_running():
            # If the loop is already running, ensure the coroutine runs within it
            return loop.run_until_complete(coro_func(*args, **kwargs))
        else:
            # If no loop is running, use asyncio.run
            return asyncio.run(coro_func(*args, **kwargs))
    except RuntimeError:
        # Handle case where the event loop is closed
        new_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(new_loop)
        return new_loop.run_until_complete(coro_func(*args, **kwargs))

async def is_feed_done(client, feed_id: str):
    response = await client.is_feed_done(feed_id)

    return response.is_feed_done.result if response.is_feed_done is not None else None

async def query_contents(client, feed_id: str, search: Optional[str] = None):
    try:
        response = await client.query_contents(
            filter=input_types.ContentFilter(
                search=search,
                searchType=enums.SearchTypes.HYBRID,
                feeds=[
                    input_types.EntityReferenceFilter(
                        id=feed_id
                    )
                ]
            )
        )

        return response.contents.results if response.contents is not None else None
    except exceptions.GraphQLClientError as e:
        print(str(e))
        return None

async def format_feed_contents(client, feed_id: str, search: Optional[str] = None):
    try:
        contents = await query_contents(client, feed_id, search)

        results = []

        for content in contents:
            results.extend(format_content(content))

        text = "\n".join(results)

        return text
    except exceptions.GraphQLClientError as e:
        print(str(e))
        raise ToolException(str(e)) from e



from abc import abstractmethod
from typing import Any, Type, Dict
from pydantic import BaseModel

class BaseTool(BaseModel):
    """
    Abstract base class for tools.
    
    Attributes:
        name (str): The name of the tool.
        description (str): A short description of the tool's functionality.
        args_schema (Type[BaseModel]): The schema for arguments expected by the tool.
    """
    name: str
    description: str
    args_schema: Type[BaseModel]

    def run(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        """
        Public method to execute the tool. Delegates to the abstract _run method.

        Args:
            *args (Any): Positional arguments passed to the tool.
            **kwargs (Any): Keyword arguments passed to the tool.

        Returns:
            Any: The result of executing the tool.
        """
        return self._run(*args, **kwargs)

    @abstractmethod
    def _run(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        """
        Abstract method to define the tool's behavior.

        Subclasses must implement this method to provide the actual logic.

        Args:
            *args (Any): Positional arguments passed to the tool.
            **kwargs (Any): Keyword arguments passed to the tool.

        Returns:
            Any: The result of executing the tool.
        """

    async def arun(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        """
        Public async method to execute the tool. Delegates to the abstract _arun method.

        Args:
            *args (Any): Positional arguments passed to the tool.
            **kwargs (Any): Keyword arguments passed to the tool.

        Returns:
            Any: The result of executing the tool asynchronously.
        """
        return await self._arun(*args, **kwargs)

    @abstractmethod
    async def _arun(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        """
        Abstract async method to define the tool's asynchronous behavior.

        Subclasses must implement this method to provide the actual async logic.

        Args:
            *args (Any): Positional arguments passed to the tool.
            **kwargs (Any): Keyword arguments passed to the tool.

        Returns:
            Any: The result of executing the tool asynchronously.
        """

    @property
    def json_schema(self) -> Dict[str, Any]:
        """Get the tool's JSON schema."""
        return self.args_schema.model_json_schema()

    def to_openai_tool(self) -> Dict[str, Any]:
        """
        Creates a tool definition compatible with OpenAI tool calling.

        Returns:
            dict: The tool definition for OpenAI.
        """
        return {
            "name": self.name,
            "description": self.description,
            "parameters": self.json_schema
        }



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class MicrosoftTeamsIngestInput(BaseModel):
    team_name: str = Field(description="Microsoft Teams team name")
    channel_name: str = Field(description="Microsoft Teams channel name")
    search: Optional[str] = Field(description="Text to search for within ingested messages", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of messages from Microsoft Teams channel to be read", default=10)

class MicrosoftTeamsIngestTool(BaseTool):
    name: str = "Graphlit Microsoft Teams ingest tool"
    description: str = """Ingests messages from Microsoft Teams channel into knowledge base.
    Optionally accepts search text for searching within the ingested messages. If search text was not provided, all ingested messages will be returned.
    Returns extracted Markdown text and metadata from messages."""
    args_schema: Type[BaseModel] = MicrosoftTeamsIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the MicrosoftTeamsIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting messages. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, team_name: Optional[str] = None, channel_name: Optional[str] = None, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        team_id = os.environ['MICROSOFT_TEAMS_TEAM_ID']
        channel_id = os.environ['MICROSOFT_TEAMS_CHANNEL_ID']

        refresh_token = os.environ['MICROSOFT_TEAMS_REFRESH_TOKEN']

        if refresh_token is None:
            raise ToolException('Invalid Microsoft Teams refresh token. Need to assign MICROSOFT_TEAMS_REFRESH_TOKEN environment variable.')

        client_id = os.environ['MICROSOFT_TEAMS_CLIENT_ID']

        if client_id is None:
            raise ToolException('Invalid Microsoft Teams client identifier. Need to assign MICROSOFT_TEAMS_CLIENT_ID environment variable.')

        client_secret = os.environ['MICROSOFT_TEAMS_CLIENT_SECRET']

        if client_secret is None:
            raise ToolException('Invalid Microsoft Teams client secret. Need to assign MICROSOFT_TEAMS_CLIENT_SECRET environment variable.')

        teams = None

        try:
            response = await self.graphlit.client.query_microsoft_teams_teams(
                properties=input_types.MicrosoftTeamsTeamsInput(
                    refreshToken=refresh_token,
                )
            )

            teams = response.microsoft_teams_teams.results if response.microsoft_teams_teams is not None else None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if len(teams) == 0:
            raise ToolException('No Microsoft Teams teams were found.')

        team = next(filter(lambda x: x['team_name'] == team_name, teams), None) if team_name is not None else teams[0]

        team_id = team.team_id if team is not None else team_id

        if team_id is None:
            raise ToolException('Invalid Microsoft Teams team identifier. Need to assign MICROSOFT_TEAMS_TEAM_ID environment variable.')

        channels = None

        try:
            response = await self.graphlit.client.query_microsoft_teams_channels(
                properties=input_types.MicrosoftTeamsChannelsInput(
                    refreshToken=refresh_token,
                ),
                team_id=team_id
            )

            channels = response.microsoft_teams_channels.results if response.microsoft_teams_channels is not None else None
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        if len(channels) == 0:
            raise ToolException('No Microsoft Teams channels were found.')

        channel = next(filter(lambda x: x['channel_name'] == channel_name, channels), None) if channel_name is not None else channels[0]

        channel_id = channel.channel_id if channel is not None else channel_id

        if channel_id is None:
            raise ToolException('Invalid Microsoft Teams channel identifier. Need to assign MICROSOFT_TEAMS_CHANNEL_ID environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='Microsoft Teams',
                    type=enums.FeedTypes.MICROSOFT_TEAMS,
                    microsoftTeams=input_types.MicrosoftTeamsFeedPropertiesInput(
                        type=enums.FeedListingTypes.PAST,
                        teamId=team_id,
                        channelId=channel_id,
                        clientId=client_id,
                        clientSecret=client_secret,
                        refreshToken=refresh_token,
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, team_name: Optional[str] = None, channel_name: Optional[str] = None, search: Optional[str] = None, read_limit: Optional[int] = None) -> str:
        return helpers.run_async(self._arun, team_name, channel_name, search, read_limit)



import logging
import time
import os
from typing import Optional, Type

from graphlit import Graphlit
from graphlit_api import exceptions, input_types, enums
from pydantic import BaseModel, Field

from ..base_tool import BaseTool
from ..exceptions import ToolException
from .. import helpers

logger = logging.getLogger(__name__)

class NotionIngestInput(BaseModel):
    search: Optional[str] = Field(description="Text to search for within ingested pages", default=None)
    read_limit: Optional[int] = Field(description="Maximum number of pages from Notion database to be read", default=10)

class NotionIngestTool(BaseTool):
    name: str = "Graphlit Notion ingest tool"
    description: str = """Ingests pages from Notion database into knowledge base.
    Optionally accepts search text for searching within the ingested pages. If search text was not provided, all ingested pages will be returned.
    Returns extracted Markdown text and metadata from Notion pages."""
    args_schema: Type[BaseModel] = NotionIngestInput

    graphlit: Graphlit = Field(None, exclude=True)

    workflow_id: Optional[str] = Field(None, exclude=True)
    correlation_id: Optional[str] = Field(None, exclude=True)

    model_config = {
        "arbitrary_types_allowed": True
    }

    def __init__(self, graphlit: Optional[Graphlit] = None, workflow_id: Optional[str] = None, correlation_id: Optional[str] = None, **kwargs):
        """
        Initializes the NotionIngestTool.

        Args:
            graphlit (Optional[Graphlit]): An optional Graphlit instance to interact with the Graphlit API.
                If not provided, a new Graphlit instance will be created.
            workflow_id (Optional[str]): ID for the workflow to use when ingesting pages. Defaults to None.
            correlation_id (Optional[str]): Correlation ID for tracking requests. Defaults to None.
            **kwargs: Additional keyword arguments for the BaseTool superclass.
        """
        super().__init__(**kwargs)
        self.graphlit = graphlit or Graphlit()
        self.workflow_id = workflow_id
        self.correlation_id = correlation_id

    async def _arun(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        feed_id = None

        token = os.environ['NOTION_API_KEY']

        if token is None:
            raise ToolException('Invalid Notion API key. Need to assign NOTION_API_KEY environment variable.')

        database_id = os.environ['NOTION_DATABASE_ID']

        if database_id is None:
            raise ToolException('Invalid Notion database identifier. Need to assign NOTION_DATABASE_ID environment variable.')

        try:
            response = await self.graphlit.client.create_feed(
                feed=input_types.FeedInput(
                    name='Notion Feed',
                    type=enums.FeedTypes.NOTION,
                    notion=input_types.NotionFeedPropertiesInput(
                        type=enums.NotionTypes.DATABASE,
                        identifiers=[database_id],
                        token=token,
                        readLimit=read_limit if read_limit is not None else 10
                    ),
                    workflow=input_types.EntityReferenceInput(id=self.workflow_id) if self.workflow_id is not None else None,
                ),
                correlation_id=self.correlation_id
            )

            feed_id = response.create_feed.id if response.create_feed is not None else None

            if feed_id is None:
                raise ToolException('Invalid feed identifier.')

            logger.debug(f'Created feed [{feed_id}].')

            # Wait for feed to complete, since ingestion happens asychronously
            done = False
            time.sleep(5)

            while not done:
                done = await helpers.is_feed_done(self.graphlit.client, feed_id)

                if done is None:
                    break

                if not done:
                    time.sleep(5)

            logger.debug(f'Completed feed [{feed_id}].')
        except exceptions.GraphQLClientError as e:
            logger.error(str(e))
            raise ToolException(str(e)) from e

        return await helpers.format_feed_contents(self.graphlit.client, feed_id, search)

    def _run(self, search: Optional[str] = None, read_limit: Optional[int] = None) -> Optional[str]:
        return helpers.run_async(self._arun, search, read_limit)
